/*
 *  include/ub/io_acct.h
 *
 *  Copyright (C) 2006 SWsoft
 *  All rights reserved.
 *  
 *  Licensing governed by "linux/COPYING.SWsoft" file.
 *
 *  Pavel Emelianov <xemul@openvz.org>
 *
 */

#ifndef __UB_IO_ACCT_H_
#define __UB_IO_ACCT_H_

#ifdef CONFIG_UBC_IO_ACCT
#include <ub/beancounter.h>

extern struct page_beancounter **page_io_pb(struct page *);

extern void ub_io_save_context(struct page *, size_t);
extern void ub_io_release_context(struct page *, size_t);

static inline void ub_io_account_read(size_t bytes)
{
	ub_percpu_add(get_exec_ub(), bytes_read, bytes);
}

static inline void ub_io_account_write(size_t bytes)
{
	ub_percpu_add(get_exec_ub(), bytes_wrote, bytes);
}

static inline void ub_io_account_dirty(struct page *page, size_t bytes)
{
	ub_io_save_context(page, bytes);
}

static inline void ub_io_account_write_cancelled(size_t bytes)
{
	ub_percpu_add(get_exec_ub(), bytes_cancelled, bytes);
}

void ub_init_io(kmem_cache_t *);
#else /* UBC_IO_ACCT */
static inline void ub_io_release_context(struct page *pg, size_t bytes)
{
}

static inline void ub_io_account_dirty(struct page *p, size_t bytes)
{
}

static inline void ub_io_account_read(size_t bytes)
{
}

static inline void ub_io_account_write(size_t bytes)
{
}

static inline void ub_io_account_write_cancelled(size_t bytes)
{
}

static inline void ub_init_io(kmem_cache_t *pb_cachep) { };
#endif

#ifdef CONFIG_UBC_DEBUG_IO
extern void ub_io_release_debug(struct page *pg);
#else
#define ub_io_release_debug(pg)	do { } while (0)
#endif
#endif
